/*
 * Free to use software from Ewald Kendziorra, 2015 - www.kendziorra.nl
 * ### ESP 8266 NTP Time Server lib - Demo timestamp with millisseconds ###
 */
#define USE_TIME
#include <ESP8266WiFi.h>
#include <NtpTime.h>
#ifdef USE_TIME
  #include <TimeLib.h>  // Arduino IDE 1.6+
#endif
#include <Streaming.h>

/*********************
 ***** EDIT THIS *****/

const char mySSID[] = "0000";
const char myPASSWORD[] = "2598089850439436";
//char mySSID[] = "edit-edit";
//char myPASSWORD[] = "edit-edit-edit-edit";
char ntpServerName[] = "nl.pool.ntp.org";  // NTP server pool (http://www.pool.ntp.org/zone)
unsigned int localPort = 8888;   // 
unsigned int syncOffset = 1800;  // time sync after each n seconds; small value = more precision
/****** END EDIT *****
 *********************/

#ifdef USE_TIME
  time_t getTime() {  // Sync Provider: sync time with NTP time
    int zone = 3600;  // +1 (MET)
    return ntp.timestamp + zone;  // UTC/GMT+MET
  }
#endif

String twoDig(int n) {
  return String(n).length() == 1 ?  "0" + String(n) : String(n); 
}

void ShowTime()
{
  Serial << 
    F("\tTime is synced  : ") << (ntp.timeIsSynced ? "true" : "false") << endl <<
    F("\tUnix timestamp  : ") << ntp.timestamp << "\t(unsigned long)" << endl << 
    F("\tmillisseconds   : ") << ntp.milliseconds << "\t\t(unsigned int)" << endl << 
    F("\tMillis timestamp: ") << ntp.msTimestamp << "\t(char*)" << endl; 
#ifdef USE_TIME
  Serial << F("\tZone timestamp  : ") << now() << endl <<
    "\t" << twoDig(day()) << "-" << twoDig(month()) << "-" << year() << " " << 
    twoDig(hour()) << ":" << twoDig(minute()) << ":" << twoDig(second()) << endl << endl;
#endif
}

void setup()
{
  Serial.begin(115200);

  ntp.localPort = localPort;
  ntp.ntpServerName = (char*)ntpServerName;
  ntp.syncOffset = syncOffset;
  
  WiFi.begin(mySSID, myPASSWORD);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial << ".";
  }
  Serial << endl << 
    F("WiFi connected") << endl << 
    F("IP address: ") << endl << 
    WiFi.localIP() << endl << 
    F("Starting UDP") << endl;
  ntp.syncTheTime();
#ifdef USE_TIME
  setSyncProvider(getTime);
  setSyncInterval(syncOffset);
#endif
}

void loop() {
  ntp.addTime();
  ShowTime();
  delay(999);  // ~one screen update/sec (1000 - ~1ms Serial print time)
}